DROP TRIGGER IF EXISTS insert_messages_trigger ON messages;
DROP FUNCTION IF EXISTS message_table_name(TIMESTAMP) CASCADE;
DROP FUNCTION IF EXISTS partition_maintenance_function() CASCADE;
DROP FUNCTION IF EXISTS create_table_function(TIMESTAMP) CASCADE;
DROP FUNCTION IF EXISTS messages_insert_trigger() CASCADE;

CREATE FUNCTION message_table_name( tbl_date TIMESTAMP )
RETURNS CHAR(30) AS $$
DECLARE
the_hour CHAR(2);
BEGIN
	the_hour := (select extract(hour from tbl_date::timestamp))::text;
	IF (the_hour::int < 10) THEN
		the_hour := '0'||the_hour::text;
	END IF;

	RETURN 'messages_' || REPLACE((tbl_date::DATE)::text, '-','_') || '_' || the_hour::text;
END;
$$
LANGUAGE plpgsql;

CREATE FUNCTION partition_maintenance_function()
RETURNS INTEGER AS $$
DECLARE
_count INTEGER;
_keep_hours INTEGER;
_lower_bound TIMESTAMP;
_result RECORD;
BEGIN
	_keep_hours := (SELECT keep_in_db FROM settings);
	_lower_bound := (localtimestamp - ( _keep_hours || ' hours')::interval)::timestamp;
	_count = 0;
	FOR _result IN (SELECT table_name from information_schema.tables
		WHERE table_catalog = 'op5lsu'
		AND table_name LIKE 'message_%') LOOP
		IF  char_length(substring(_result.table_name from '[0-9_]*$')) <> 0 AND
			(_lower_bound > to_timestamp(substring(_result.table_name from '[0-9_]*$'), '_YYYY_MM_DD_HH24')) THEN
			EXECUTE 'DROP TABLE ' || quote_ident(_result.table_name);
			_count = _count + 1;
		END IF;
	END LOOP;
	RETURN _count;
END
$$
LANGUAGE plpgsql;

CREATE FUNCTION create_table_function( tbl_date TIMESTAMP )
RETURNS CHAR(30) AS $$
DECLARE
table_name CHAR(30);
_hour CHAR(2);
_starttime timestamp;
_endtime timestamp;
BEGIN
	table_name := message_table_name(tbl_date);
	IF EXISTS (
		SELECT *
		FROM   pg_catalog.pg_tables 
		WHERE    tablename  = table_name
		) THEN
		RETURN table_name;
	ELSE
		_hour := (select extract(hour from tbl_date::timestamp))::text;
		_starttime := (select tbl_date::date::text || ' ' || _hour || ':00' )::timestamp;
		_endtime := (select tbl_date::date::text || ' ' || _hour || ':00' )::timestamp + interval '1 hour'; 
		EXECUTE ( 'CREATE TABLE '
			|| quote_ident(table_name)
			|| ' ( CHECK ( rtime < '
			|| quote_literal(_endtime)
			|| ' AND rtime >= '
			|| quote_literal(_starttime)
			|| '), PRIMARY KEY (id, rtime)
		) INHERITS (messages);' );
		PERFORM partition_maintenance_function();
	END IF;
	RETURN table_name;
END;
$$
LANGUAGE plpgsql;

CREATE FUNCTION messages_insert_trigger()
RETURNS TRIGGER AS $$
DECLARE
table_name CHAR(30);
BEGIN
	table_name := create_table_function(NEW.rtime);
	EXECUTE 'INSERT INTO ' ||  quote_ident(table_name) || ' (ip, rtime, facility, severity, mtime, host, ident, pid, event, msg, fts) VALUES($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11);'
		USING NEW.ip, NEW.rtime, NEW.facility, NEW.severity, NEW.mtime, NEW.host, NEW.ident, NEW.pid, NEW.event, NEW.msg, NEW.fts;
	RETURN NULL;
END;
$$
LANGUAGE plpgsql;


CREATE TRIGGER insert_messages_trigger
	BEFORE INSERT ON messages
	FOR EACH ROW EXECUTE PROCEDURE messages_insert_trigger();


--UPDATE db VERSION
UPDATE service SET version = 20;

